extends Node2D


# Level Variables
@export var level_seed = -1
var platform = FastNoiseLite.new()
var width  = 12
var height = 8

# Components
@onready var player    = get_node("../Player").get_child(0)
@onready var particles = get_node("Particles")
@onready var tilemap_main   = get_node("TileMap_Main")

# Items
@onready var particle_instance = preload("res://src/effects/particles.tscn")

# Cell Registry
var tile_reg = []



func _ready():
	if level_seed < 0:
		platform.seed = randi()
	else:
		platform.seed = level_seed
	platform.frequency = 0.1
	
	for x in range(-width/2, width/2):
		for y in range(-height/2, height/2):
			tile_reg.append(Vector2i(x, y))



func _process(delta):
	generate_chunk(player.position)



func generate_chunk(position):
	var tile_pos = tilemap_main.local_to_map(position)
	for x in range(-width, width):
		for y in range(-height, height):
			if not check_cell(Vector2i(tile_pos.x+x, tile_pos.y+y)):
				var cell_noise = platform.get_noise_2d(tile_pos.x + x, tile_pos.y + y)
				if cell_noise > 0.2:
					generate_cell(Vector2i(tile_pos.x+x, tile_pos.y+y))
				#if cell_noise < 0.0001:
				#	generate_particles(position.x+(x*16), position.y+(y*16))



# Check if the cell has already been set
func check_cell(tile_cords):
	if tile_reg.find(tile_cords) == -1:
		tile_reg.append(tile_cords)
		return false
	else:
		return true



func generate_cell(tile_cords):
	tilemap_main.set_cells_terrain_connect(
		0,
		[Vector2i(tile_cords.x, tile_cords.y)],
		0,
		0,
		true
	)



func generate_particles(pos_x, pos_y):
		var new_particle = particle_instance.instantiate()
		particles.add_child(new_particle)
		new_particle.position = Vector2(pos_x, pos_y)
	
