extends CharacterBody2D


const SPEED = 150.0
const JUMP_VELOCITY = -400.0
var gravity = ProjectSettings.get_setting("physics/2d/default_gravity")
var dash_energy = true


@onready var animated_sprite = $AnimatedSprite2D
@onready var particles = $CPUParticles2D


func _physics_process(delta):
	# Add the gravity.
	
	var flash_range = animated_sprite.material.get_shader_parameter("flash_range") * 0.8
	animated_sprite.material.set_shader_parameter("flash_range", flash_range)
	
	if not is_on_floor():
		velocity.y += gravity * delta
		dash_energy = true
		
		if velocity.y <= 0:
			animated_sprite.animation = "jump"
		else:
			animated_sprite.animation = "fall"
	else:
		if velocity.x == 0:
			animated_sprite.animation = "idle"
		else:
			animated_sprite.animation = "run"
		

	# Handle Jump.
	if Input.is_action_just_pressed("move_jump") and is_on_floor():
		velocity.y = JUMP_VELOCITY
		flash()
	
	# Handel Dash.
	if Input.is_action_just_pressed("move_dash") and dash_energy:
		dash_energy = false
		particles.emitting = true
		

	# Get the input direction and handle the movement/deceleration.
	# As good practice, you should replace UI actions with custom gameplay actions.
	var direction = Input.get_axis("move_left", "move_right")
	if direction:
		velocity.x = direction * SPEED
		if velocity.x < 0:
			animated_sprite.flip_h = true
		elif velocity.x > 0:
			animated_sprite.flip_h = false  
	else:
		velocity.x = move_toward(velocity.x, 0, SPEED)

	move_and_slide()


func flash():
	animated_sprite.material.set_shader_parameter("flash_range", 1.0)
	
